@@ -1,36 +1,40 @@
 import logging
 
-from typing import Optional, Dict, Any, Tuple, List, Union
+from typing import Optional, Dict, Any, Tuple, List, Union, Callable
 from os import environ, path
 from hashlib import sha1 as sha1_hash
-from enum import Enum
 from dataclasses import dataclass, field
+from abc import ABCMeta
 
 import yaml
 
 from behave.model import Scenario
-from jinja2.environment import Template
+from locust.user.sequential_taskset import SequentialTaskSet
+from locust.env import Environment
+
+from .types import GrizzlyDict
 
-from .testdata.models import TemplateData
-from .types import ResponseContentType, HandlerType, RequestMethod
 
 logger = logging.getLogger(__name__)
 
 
+# @TODO: these two classes is needed here to avoid circular imports
+class GrizzlyTasksBase(SequentialTaskSet):
+    pass
+
+
+@dataclass(unsafe_hash=True)
+class GrizzlyTask(metaclass=ABCMeta):
+    scenario: 'GrizzlyContextScenario' = field(init=False, repr=False)
+
+    def implementation(self) -> Callable[[GrizzlyTasksBase], Any]:
+        raise NotImplementedError(f'{self.__class__.__name__} has not implemented "implementation"')
+
+
 def generate_identifier(name: str) -> str:
     return sha1_hash(name.encode('utf-8')).hexdigest()[:8]
 
 
-class ResponseTarget(Enum):
-    METADATA = 0
-    PAYLOAD = 1
-
-
-class ResponseAction(Enum):
-    VALIDATE = 0
-    SAVE = 1
-
-
 def load_configuration_file() -> Dict[str, Any]:
     def _flatten(node: Dict[str, Any], parents: Optional[List[str]] = None) -> Dict[str, Any]:
         flat: Dict[str, Any] = {}
@@ -67,35 +71,37 @@ def load_configuration_file() -> Dict[str, Any]:
 
 
 @dataclass
-class LocustContextState:
+class GrizzlyContextState:
     spawning_complete: bool = field(default=False)
     background_section_done: bool = field(default=False)
-    variables: TemplateData = field(init=False, default_factory=TemplateData)
+    variables: GrizzlyDict = field(init=False, default_factory=GrizzlyDict)
     configuration: Dict[str, Any] = field(init=False, default_factory=load_configuration_file)
     alias: Dict[str, str] = field(init=False, default_factory=dict)
+    verbose: bool = field(default=False)
+    environment: Environment = field(init=False, repr=False)
 
 
 @dataclass
-class LocustContextScenarioWait:
+class GrizzlyContextScenarioWait:
     minimum: float = field(default=1.0)
     maximum: float = field(default=1.0)
 
 
 @dataclass
-class LocustContextScenarioResponseTimePercentile:
+class GrizzlyContextScenarioResponseTimePercentile:
     response_time: int
     percentile: float
 
 
 @dataclass(unsafe_hash=True)
-class LocustContextScenarioValidation:
+class GrizzlyContextScenarioValidation:
     fail_ratio: Optional[float] = field(init=False, default=None)
     avg_response_time: Optional[int] = field(init=False, default=None)
-    response_time_percentile: Optional[LocustContextScenarioResponseTimePercentile] = field(init=False, default=None)
+    response_time_percentile: Optional[GrizzlyContextScenarioResponseTimePercentile] = field(init=False, default=None)
 
 
 @dataclass(unsafe_hash=True)
-class LocustContextScenario:
+class GrizzlyContextScenario:
     name: str = field(init=False, hash=True)
     user_class_name: str = field(init=False, hash=True)
     _identifier: Optional[str] = field(init=False, hash=True, default=None)
@@ -103,9 +109,9 @@ class LocustContextScenario:
 
     behave: Scenario = field(init=False, repr=False, hash=False, compare=False)
     context: Dict[str, Any] = field(init=False, repr=False, hash=False, compare=False, default_factory=dict)
-    wait: LocustContextScenarioWait = field(init=False, repr=False, hash=False, compare=False, default_factory=LocustContextScenarioWait)
-    tasks: List[Union['RequestContext', float]] = field(init=False, repr=False, hash=False, compare=False, default_factory=list)
-    validation: LocustContextScenarioValidation = field(init=False, hash=False, compare=False, default_factory=LocustContextScenarioValidation)
+    wait: GrizzlyContextScenarioWait = field(init=False, repr=False, hash=False, compare=False, default_factory=GrizzlyContextScenarioWait)
+    tasks: List[GrizzlyTask] = field(init=False, repr=False, hash=False, compare=False, default_factory=list)
+    validation: GrizzlyContextScenarioValidation = field(init=False, hash=False, compare=False, default_factory=GrizzlyContextScenarioValidation)
     stop_on_failure: bool = field(init=False, default=False)
     orphan_templates: List[str] = field(init=False, repr=False, hash=False, compare=False, default_factory=list)
 
@@ -135,19 +141,19 @@ class LocustContextScenario:
             self.validation.response_time_percentile is not None
         )
 
-    def add_task(self, request_context: Union['RequestContext', float]) -> None:
-        if isinstance(request_context, RequestContext) and (
-            not hasattr(request_context, 'scenario') or
-            request_context.scenario is None or
-            request_context.scenario is not self
+    def add_task(self, task: GrizzlyTask) -> None:
+        if isinstance(task, GrizzlyTask) and (
+            not hasattr(task, 'scenario') or
+            task.scenario is None or
+            task.scenario is not self
         ):
-            request_context.scenario = self
+            task.scenario = self
 
-        self.tasks.append(request_context)
+        self.tasks.append(task)
 
 
 @dataclass
-class LocustContextSetup:
+class GrizzlyContextSetup:
     log_level: str = field(init=False, default='INFO')
 
     global_context: Dict[str, Any] = field(init=False, repr=False, hash=False, compare=False, default_factory=dict)
@@ -159,16 +165,16 @@ class LocustContextSetup:
     statistics_url: Optional[str] = field(init=False, default=None)
 
 
-class LocustContext:
-    __instance: Optional['LocustContext'] = None
+class GrizzlyContext:
+    __instance: Optional['GrizzlyContext'] = None
 
     _initialized: bool
-    _state: LocustContextState
-    _setup: LocustContextSetup
-    _scenarios: List[LocustContextScenario]
+    _state: GrizzlyContextState
+    _setup: GrizzlyContextSetup
+    _scenarios: List[GrizzlyContextScenario]
 
     @classmethod
-    def __new__(cls, *_args: Tuple[Any, ...], **_kwargs: Dict[str, Any]) -> 'LocustContext':
+    def __new__(cls, *_args: Tuple[Any, ...], **_kwargs: Dict[str, Any]) -> 'GrizzlyContext':
         if cls.__instance is None:
             cls.__instance = super().__new__(cls)
             cls.__instance._initialized = False
@@ -184,28 +190,28 @@ class LocustContext:
 
     def __init__(self) -> None:
         if not self._initialized:
-            self._state = LocustContextState()
-            self._setup = LocustContextSetup()
+            self._state = GrizzlyContextState()
+            self._setup = GrizzlyContextSetup()
             self._scenarios = []
             self._initialized = True
 
     @property
-    def setup(self) -> LocustContextSetup:
+    def setup(self) -> GrizzlyContextSetup:
         return self._setup
 
     @property
-    def state(self) -> LocustContextState:
+    def state(self) -> GrizzlyContextState:
         return self._state
 
     @property
-    def scenario(self) -> LocustContextScenario:
+    def scenario(self) -> GrizzlyContextScenario:
         if len(self._scenarios) < 1:
-            self._scenarios.append(LocustContextScenario())
+            self._scenarios.append(GrizzlyContextScenario())
 
         return self._scenarios[-1]
 
     def add_scenario(self, source: Union[Scenario, str]) -> None:
-        scenario = LocustContextScenario()
+        scenario = GrizzlyContextScenario()
         if isinstance(source, Scenario):
             name = source.name
             scenario.behave = source
@@ -215,56 +221,12 @@ class LocustContext:
         scenario.name = name
         self._scenarios.append(scenario)
 
-    def scenarios(self) -> List[LocustContextScenario]:
+    def scenarios(self) -> List[GrizzlyContextScenario]:
         return self._scenarios
 
-    def get_scenario(self, name: str) -> Optional[LocustContextScenario]:
+    def get_scenario(self, name: str) -> Optional[GrizzlyContextScenario]:
         for scenario in self._scenarios:
             if scenario.get_name() == name:
                 return scenario
 
         return None
-
-@dataclass(unsafe_hash=True)
-class RequestContextHandlers:
-    metadata: List[HandlerType] = field(init=False, hash=False, default_factory=list)
-    payload: List[HandlerType] = field(init=False, hash=False, default_factory=list)
-
-    def add_metadata(self, handler: HandlerType) -> None:
-        self.metadata.append(handler)
-
-    def add_payload(self, handler: HandlerType) -> None:
-        self.payload.append(handler)
-
-
-@dataclass(unsafe_hash=True)
-class RequestContextResponse:
-    status_codes: List[int] = field(init=False, repr=False, hash=False, default_factory=list)
-    content_type: ResponseContentType = field(init=False, repr=False, default=ResponseContentType.GUESS)
-    handlers: RequestContextHandlers = field(init=False, repr=False, default_factory=RequestContextHandlers)
-
-    def __post_init__(self) -> None:
-        if 200 not in self.status_codes:
-            self.status_codes.append(200)
-
-    def add_status_code(self, status: int) -> None:
-        absolute_status = abs(status)
-        if absolute_status not in self.status_codes or status not in self.status_codes:
-            if absolute_status == status:
-                self.status_codes.append(status)
-            else:
-                index = self.status_codes.index(absolute_status)
-                self.status_codes.pop(index)
-
-
-@dataclass(unsafe_hash=True)
-class RequestContext:
-    method: RequestMethod
-    name: str
-    endpoint: str
-    scenario: LocustContextScenario = field(init=False, repr=False)
-    template: Optional[Template] = field(init=False, repr=False, default=None)
-    source: Optional[str] = field(init=False, repr=False, default=None)
-
-    response: RequestContextResponse = field(init=False, repr=False, default_factory=RequestContextResponse)
-
