@@ -59,17 +59,16 @@ from typing import Dict, List, Any, Type, Optional, cast
 from csv import DictReader
 from random import randint
 
-from ...types import bool_typed
-from . import AtomicVariable, parse_arguments
+from ...types import bool_typed, AtomicVariable
 
 
 def atomiccsvrow__base_type__(value: str) -> str:
-    locust_context_requests = os.path.join(os.environ.get('LOCUST_CONTEXT_ROOT', ''), 'requests')
+    grizzly_context_requests = os.path.join(os.environ.get('GRIZZLY_CONTEXT_ROOT', ''), 'requests')
 
     if '|' in value:
         [csv_file, csv_arguments] = [v.strip() for v in value.split('|', 1)]
 
-        arguments = parse_arguments(AtomicCsvRow, csv_arguments)
+        arguments = AtomicCsvRow.parse_arguments(csv_arguments)
 
         for argument, value in arguments.items():
             if argument not in AtomicCsvRow.arguments:
@@ -81,13 +80,13 @@ def atomiccsvrow__base_type__(value: str) -> str:
     else:
         csv_file = value
 
-    path = os.path.join(locust_context_requests, csv_file)
+    path = os.path.join(grizzly_context_requests, csv_file)
 
     if not path.endswith('.csv'):
         raise ValueError(f'AtomicCsvRow: {csv_file} must be a CSV file with file extension .csv')
 
     if not os.path.isfile(path):
-        raise ValueError(f'AtomicCsvRow: {csv_file} is not a file in {locust_context_requests}')
+        raise ValueError(f'AtomicCsvRow: {csv_file} is not a file in {grizzly_context_requests}')
 
     return value
 
@@ -112,7 +111,7 @@ class AtomicCsvRow(AtomicVariable[Dict[str, Any]]):
         if '|' in safe_value:
             [csv_file, csv_arguments] = [v.strip() for v in safe_value.split('|', 1)]
 
-            arguments = parse_arguments(self.__class__, csv_arguments)
+            arguments = self.parse_arguments(csv_arguments)
 
             for argument, caster in self.__class__.arguments.items():
                 if argument in arguments:
@@ -132,7 +131,7 @@ class AtomicCsvRow(AtomicVariable[Dict[str, Any]]):
 
             return
 
-        self.context_root = os.path.join(os.environ.get('LOCUST_CONTEXT_ROOT', ''), 'requests')
+        self.context_root = os.path.join(os.environ.get('GRIZZLY_CONTEXT_ROOT', ''), 'requests')
         self._rows = {variable: self._create_row_queue(csv_file)}
         self._settings = {variable: settings}
         self.__initialized = True
